﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2011 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Linq;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Optimization.Operators.LCS {
  [Item("BestSolutionCreator", "Description missing")]
  [StorableClass]
  public class BestSolutionCreator : SingleSuccessorOperator {

    #region Parameter Properties
    public IValueLookupParameter<IGAssistSolution> SolutionParameter {
      get { return (IValueLookupParameter<IGAssistSolution>)Parameters["Solution"]; }
    }
    public ScopeTreeLookupParameter<IGAssistIndividual> IndividualParameter {
      get { return (ScopeTreeLookupParameter<IGAssistIndividual>)Parameters["Individual"]; }
    }
    public LookupParameter<IGAssistNichesProblemData> ProblemDataParameter {
      get { return (LookupParameter<IGAssistNichesProblemData>)Parameters["ProblemData"]; }
    }
    public IScopeTreeLookupParameter<DoubleValue> QualityParameter {
      get { return (IScopeTreeLookupParameter<DoubleValue>)Parameters["Quality"]; }
    }
    public ILookupParameter<BoolValue> MaximizationParameter {
      get { return (ILookupParameter<BoolValue>)Parameters["Maximization"]; }
    }
    #endregion

    [StorableConstructor]
    protected BestSolutionCreator(bool deserializing) : base(deserializing) { }
    public BestSolutionCreator()
      : base() {
      Parameters.Add(new ValueLookupParameter<IGAssistIndividual>("Solution", ""));
      Parameters.Add(new ScopeTreeLookupParameter<IGAssistIndividual>("Individual", ""));
      Parameters.Add(new LookupParameter<IGAssistNichesProblemData>("ProblemData", ""));
      Parameters.Add(new ScopeTreeLookupParameter<DoubleValue>("Quality", "The qualities of the trees that should be analyzed."));
      Parameters.Add(new LookupParameter<BoolValue>("Maximization", "The direction of optimization."));
    }

    public override IOperation Apply() {
      var qualities = QualityParameter.ActualValue;
      var bestQuality = MaximizationParameter.ActualValue.Value ? qualities.Max() : qualities.Min();
      int index = qualities.IndexOf(bestQuality);
      SolutionParameter.ActualValue = IndividualParameter.ActualValue[index].CreateGAssistSolution(ProblemDataParameter.ActualValue);
      return base.Apply();
    }
  }
}
